//  This file was generated by LevelHelper
//  http://levelhelper.wordpress.com
//
//  LevelHelperLoader.h - online version
//  Created by Bogdan Vladu
//  Copyright 2011 Bogdan Vladu. All rights reserved.
//
////////////////////////////////////////////////////////////////////////////////////////////////////////
//
//  This software is provided 'as-is', without any express or implied
//  warranty.  In no event will the authors be held liable for any damages
//  arising from the use of this software.
//  Permission is granted to anyone to use this software for any purpose,
//  including commercial applications, and to alter it and redistribute it
//  freely, subject to the following restrictions:
//  The origin of this software must not be misrepresented; you must not
//  claim that you wrote the original software. If you use this software
//  in a product, an acknowledgment in the product documentation would be
//  appreciated but is not required.
//  Altered source versions must be plainly marked as such, and must not be
//  misrepresented as being the original software.
//  This notice may not be removed or altered from any source distribution.
//  By "software" the author refers to this code file and not the application 
//  that was used to generate this file.
//
////////////////////////////////////////////////////////////////////////////////////////////////////////
//
//  Version history
//  v1.0 first draft
//  v1.1 added removeAllCCSprites, removeAllObjects, subfolder for levels (images have to wait)
//		 added support for colors, order Z,
//		 fix bug when having polygon shape on a sprite and scale
//	v1.2 added revolute joints, added fixed rotation on a body
//  v1.3 added support for no physic sprites
//       changed the way level is loaded to counteract some eventual bugs
//		 changed some methods result of "no physic sprites"
//  v1.4 changed the way level is loaded - now uses the popular plist format
//  v1.5 added hasWorldBoundaries method
//  v1.6 changed the way batch nodes are loaded to support background
//  v1.7 changed createWorldBoundaries method to work on both mac and ios
//  v1.8 added uniqueName as userData for CCSprite
//  v1.9 fixed release issues
//  v2.0 added method isSpriteWithUniqueName: atPoint:
//  v2.1 added support for retina display
//  v2.2 added support to load a level from internet
//  v2.2.1 added method setCustomAttributes - more info about this soon
//  v2.3 fixed retinaDisplay issue. 
//       Now level will be converted to the used device
//  v2.4 fixed world boundaries not being converted - added method dontConvertLevel
////////////////////////////////////////////////////////////////////////////////////////////////////////

#import <Foundation/Foundation.h>
#import "cocos2d.h"
#import "Box2D.h"

enum LevelHelper_TAG 
{
	DEFAULT_TAG 	= 0,
	NUMBER_OF_TAGS 	= 1
};

#define BATCH_NODE_CAPACITY 100 //you should change this value if you have more then 100 sprites in a texture image

//Pixel to metres ratio. Box2D uses metres as the unit for measurement.
//This ratio defines how many pixels correspond to 1 Box2D "metre"
//Box2D is optimized for objects of 1x1 metre therefore it makes sense
//to define the ratio so that your most common object type is 1x1 metre.
#define PTM_RATIO 32 //you should only keep this here

@protocol LevelHelperLoaderCustomCCSprite
@optional

-(CCSprite*) spriteFromDictionary:(NSDictionary*)spriteProp;

-(CCSprite*) spriteWithBatchFromDictionary:(NSDictionary*)spriteProp 
								   batchNode:(CCSpriteBatchNode*)batch;

-(void) removeFromBatchNode:(CCSprite*)sprite;


//this two methods should be overloaded together
//first one is for physic sprites 
-(void) setCustomAttributesForPhysics:(NSDictionary*)spriteProp 
                       body:(b2Body*)body
                     sprite:(CCSprite*)sprite;
//this second one is for the case where you dont use physics or you have sprites
//with "NO PHYSIC" as physic type
-(void) setCustomAttributesForNonPhysics:(NSDictionary*)spriteProp 
                               sprite:(CCSprite*)sprite;


@end

@interface LevelHelperLoader : NSObject<LevelHelperLoaderCustomCCSprite> {
	
	NSArray* lhSprites;	//array of NSDictionary with keys GeneralProperties (NSDictionary) 
						//and PhysicProperties (NSDictionary)
	NSArray* lhJoints;	//array of NSDictionary

	NSMutableDictionary* ccSpritesInScene;	//key - uniqueSpriteName	value - CCSprite* or NSValue with b2Body*
	NSMutableDictionary* noPhysicSprites;   //key - uniqueSpriteName    value - CCSprite*
	NSMutableDictionary* ccJointsInScene;   //key - uniqueJointName     value - NSValue withPointer of b2Joint*
	NSMutableDictionary* batchNodes;		//key - textureName			value - NSDictionary
	CGRect worldBoundaries;
	bool addSpritesToLayerWasUsed;
	bool addObjectsToWordWasUsed;
	
	CCLayer* cocosLayer; //hold pointer to properly release the sprites
    
    bool convertLevel;
    CGPoint convertRatio;
}

///////////////////////////////////////////////////////////////////////////////////////////////////

-(id) initWithContentOfFile:(NSString*)levelFile;

-(id) initWithContentOfFileFromInternet:(NSString*)webAddress;

-(id) initWithContentOfFile:(NSString*)levelFile 
			 levelSubfolder:(NSString*)levelFolder;

//this will convert level to the used devide - safe frame must be the size of the
//device resolution inside LevelHelper - by default convertLevel is TRUE
//only call this method do deactivate the convert level option
-(void) convertLevel:(bool)value;

///////////////////////////////////////////////////////////////////////////////////////////////////

-(void) addObjectsToWorld:(b2World*)world cocos2dLayer:(CCLayer*)cocosLayer;

///////////////////////////////////////////////////////////////////////////////////////////////////
 
-(void) addSpritesToLayer:(CCLayer*)cocosLayer;

///////////////////////////////////////////////////////////////////////////////////////////////////

-(bool) hasWorldBoundaries;

///////////////////////////////////////////////////////////////////////////////////////////////////

-(void) createWorldBoundaries:(b2World*)world;

///////////////////////////////////////////////////////////////////////////////////////////////////

-(unsigned int) numberOfBatchNodesUsed;

///////////////////////////////////////////////////////////////////////////////////////////////////

-(bool) isSpriteWithUniqueName:(NSString*)name atPoint:(CGPoint)point;

///////////////////////////////////////////////////////////////////////////////////////////////////
 
-(CCSprite*) spriteWithUniqueName:(NSString*)name; 

///////////////////////////////////////////////////////////////////////////////////////////////////

-(b2Body*) bodyWithUniqueName:(NSString*)name;

///////////////////////////////////////////////////////////////////////////////////////////////////

-(CCSprite*) newSpriteWithUniqueName:(NSString*)name 
						  cocos2dLayer:(CCLayer*)cocosLayer; 

///////////////////////////////////////////////////////////////////////////////////////////////////

-(b2Body*) newBodyWithUniqueName:(NSString*)name 
						   world:(b2World*)world 
					cocos2dLayer:(CCLayer*)cocosLayer;

///////////////////////////////////////////////////////////////////////////////////////////////////

-(NSMutableArray*) spritesWithTag:(LevelHelper_TAG)tag;

///////////////////////////////////////////////////////////////////////////////////////////////////

-(NSMutableArray*) bodiesWithTag:(LevelHelper_TAG)tag;

///////////////////////////////////////////////////////////////////////////////////////////////////

-(NSMutableArray*) newSpritesWithTag:(LevelHelper_TAG)tag
						  cocos2dLayer:(CCLayer*)cocosLayer;

///////////////////////////////////////////////////////////////////////////////////////////////////

-(NSMutableArray*) newBodiesWithTag:(LevelHelper_TAG)tag 
							  world:(b2World*)world 
					   cocos2dLayer:(CCLayer*)cocosLayer;

///////////////////////////////////////////////////////////////////////////////////////////////////

-(bool) removeSpriteWithUniqueName:(NSString*)name;

///////////////////////////////////////////////////////////////////////////////////////////////////

-(bool) removeSprite:(CCSprite*)ccsprite;

///////////////////////////////////////////////////////////////////////////////////////////////////

-(bool) removeAllSprites;

///////////////////////////////////////////////////////////////////////////////////////////////////

-(bool) removeBodyWithUniqueName:(NSString*)name;

///////////////////////////////////////////////////////////////////////////////////////////////////

-(bool) removeBody:(b2Body*)body;

///////////////////////////////////////////////////////////////////////////////////////////////////

-(bool) removeAllBodies;

///////////////////////////////////////////////////////////////////////////////////////////////////

-(b2Joint*) jointWithUniqueName:(NSString*)name;

///////////////////////////////////////////////////////////////////////////////////////////////////

-(bool) removeJointWithUniqueName:(NSString*)name;

///////////////////////////////////////////////////////////////////////////////////////////////////

-(bool) removeJoint:(b2Joint*) joint;

///////////////////////////////////////////////////////////////////////////////////////////////////

-(bool) removeAllJoints;

///////////////////////////////////////////////////////////////////////////////////////////////////

-(void) setSpriteProperties:(CCSprite*)ccsprite spriteProperties:(NSDictionary*)spriteProp;

///////////////////////////////////////////////////////////////////////////////////////////////////

@end





